package cqcli::cqtm;


use strict;

require Exporter;

use Error qw(:try);
use cqcli::Exception;
use Inline::Java qw(caught cast);
use cqcli::Util;
use cqcli::IOHandler;
use cqcli::commonservice;
use Text::ParseWords;
use Getopt::Long 2.34;


our @ISA = qw(Exporter);

our %EXPORT_TAGS = ( 'all' => [ qw() ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(tmexecute tmcommit);

our $VERSION = '0.01';

#######################################################################
our @ARGVBAACKUP = @ARGV;
our %RepetitionCount = ();
our $isLoginOpt = undef;
our $CliSession = undef;
our $JavaReqObj = undef;
our %CmdInfo = ( 'cmdstring' => [ ], 'cmdname'  => "");
our %CmdLogin = ('database'	=>	undef,'schema'=>  undef,
			 'user'		=>	undef,'password'=>  undef);
our $BLANK_MARKER_SET = 0;
our $iteration = undef;
our @RIdDef = undef;
our $computer="";
our $groups="";
our $TypeDef = "cqtm";
our $commit = 0;
our $buildname = undef;
our $buildid = undef;
our $viewpathname = undef;
our $logname = undef;
our $startupdir = undef;
our $ids = "";
our $idSpecified = 0;

#########################################################################

#########handle getopt warnings begin########################

sub InitParser_tmexecute
{
	my @tmpARGV = @_;
	@ARGVBAACKUP = @ARGV;
	@ARGV = undef;
	$isLoginOpt = undef;
	%RepetitionCount = ();
	%CmdInfo = ('cmdstring' => [ ], 'cmdname'  => "tmexecute");
	%CmdLogin =('database'	=>undef,'schema'=>  undef,
				'user'	=>	undef,'password'=>  undef);


	$CmdInfo{'cmdstring'} =[@tmpARGV];
	($CmdInfo{'cmdname'}  , @ARGV ) = @{$CmdInfo{'cmdstring'}};
	$CliSession = GetSession();
	$JavaReqObj = GetJavaHash();
	$iteration = undef;
	@RIdDef = undef;
	$computer=undef;
	$groups=undef;
	$TypeDef = "cqtm";
	$buildname = undef;
	$buildid = undef;
	$viewpathname = undef;
	$logname = undef;
	$startupdir = undef;
	$ids = "";
	$commit=0;
	$idSpecified = 0;
	return 1;
}

sub InitParser_tmcommit
{
	my @tmpARGV = @_;
	@ARGVBAACKUP = @ARGV;
	@ARGV = undef;
	$isLoginOpt = undef;
	%RepetitionCount = ();
	%CmdInfo = ('cmdstring' => [ ], 'cmdname'  => "tmcommit");
	%CmdLogin =('database'	=>undef,'schema'=>  undef,
				'user'	=>	undef,'password'=>  undef);

	$CmdInfo{'cmdstring'} =[@tmpARGV];
	($CmdInfo{'cmdname'}  , @ARGV ) = @{$CmdInfo{'cmdstring'}};
	$CliSession = GetSession();
	$JavaReqObj = GetJavaHash();
	$iteration = undef;
	@RIdDef = undef;
	$computer=undef;
	$groups=undef;
	$TypeDef = "cqtm";
	$buildname = undef;
	$buildid = undef;
	$viewpathname = undef;
	$logname = undef;
	$startupdir = undef;
	$ids = "";
	$commit=0;
      $idSpecified = 0;
	return 1;
}

sub CleanUpParser_tmexecute()
{

	@ARGV = @ARGVBAACKUP;
	%RepetitionCount = ();
	%CmdInfo = ('cmdstring' => [ ], 'cmdname'  => "tmexecute");
	%CmdLogin =('database'	=>undef,'schema'=>  undef,
				'user'	=>	undef,'password'=>  undef);


	$CmdInfo{'cmdstring'} = undef;
	$CmdInfo{'cmdname'} =undef;
	$CliSession = undef;
	$JavaReqObj = undef;
	$iteration = undef;
	@RIdDef = undef;
	$computer=undef;
	$groups=undef;
	$TypeDef = "cqtm";
	$buildname = undef;
	$buildid = undef;
	$viewpathname = undef;
	$logname = undef;
	$startupdir = undef;
      $ids = "";
	$idSpecified = 0;
	$commit=0;
}

sub CleanUpParser_tmcommit()
{

	@ARGV = @ARGVBAACKUP;
	%RepetitionCount = ();
	%CmdInfo = ('cmdstring' => [ ], 'cmdname'  => "tmcommit");
	%CmdLogin =('database'	=>undef,'schema'=>  undef,
				'user'	=>	undef,'password'=>  undef);


	$CmdInfo{'cmdstring'} = undef;
	$CmdInfo{'cmdname'} =undef;
	$CliSession = undef;
	$JavaReqObj = undef;
	$iteration = undef;
	@RIdDef = undef;
	$computer=undef;
	$groups=undef;
	$TypeDef = "cqtm";
	$buildname = undef;
	$buildid = undef;
	$viewpathname = undef;
	$logname = undef;
	$startupdir = undef;
	$ids = "";
	$idSpecified = 0;
	$commit=0;
}

#########handle getopt warnings end#####################################

sub handle_help_usage
{
	my $opt = $_[0];
	if($opt eq "help")	{
		ThrowUsageException($CmdInfo{'cmdname'},$opt);
	}
	else{
		ThrowUsageException($CmdInfo{'cmdname'},$opt);
	}
	return 1;
}

sub handle_ask_mode
{
	my $mode = shift;
	SetCQCLIAskMode(1);
	return 1;
}


sub handle_login_data
{
	my ($opt , $val) = @_;
	my $msg = undef;
	$isLoginOpt = 1;

	if($RepetitionCount{$opt}) {
		ThrowSyntaxErrorException("CQCLI_Repeat_Option","-$opt", $CmdInfo{'cmdname'} );
	}else {
		$RepetitionCount{$opt} = 1;
	}

	if( ($_[0] eq "database") && ($val)	){
		$CmdLogin{'database'} = $val;
	}elsif(($_[0] eq "user") && ($val)) {
		$CmdLogin{'user'} = $val;
	}elsif(($_[0] eq "password") && ($val)) {
		$CmdLogin{'password'} = $val;
	}elsif(($_[0] eq "password") && ($val eq "")){
		$CmdLogin{'password'} = "";
		$BLANK_MARKER_SET = 1 ;
	}
	elsif(($_[0] eq "schema") && ($val) ) {
		$CmdLogin{'schema'} = $val; 
	}else{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt",$CmdInfo{'cmdname'});
	}
	return 1;
}

sub handle_iteration_name
{
	my ($opt , $val) = @_;
	my $msg = undef;

	if($RepetitionCount{'handle_iteration_name'}) {
		#ThrowSyntaxErrorException("CQCLI_Repeat_Action","[$ActionDef and $opt ]", $CmdInfo{'cmdname'} );
	}else {
		$RepetitionCount{'handle_action_name'} = 1;
	}


	if( ($opt eq "iteration") && ($val)	){
		$iteration=$val;
	}else{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt",$CmdInfo{'cmdname'});
	}

	return 1;

}


sub handle_recordid_list
{
	my ($opt , @val) = @_;

	if( ($opt eq "id") && (@val) ){
		@RIdDef = split(/,/,join(',',@val));
	}
	else{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt",$CmdInfo{'cmdname'});
	}
	return 1;

}

sub handle_commit_option
{
	$commit = 1;
}

sub handle_build_name
{
	my ($opt , $val) = @_;

	if( ($opt eq "bname") && ($val) ){
		$buildname = $val;
	}
	else{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt",$CmdInfo{'cmdname'});
	}
}

sub handle_build_id
{
		my ($opt , $val) = @_;

	if( ($opt eq "brecord") && ($val) ){
		$buildid=$val;
	}
	else{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt",$CmdInfo{'cmdname'});
	}
}

sub handle_computer_names
{
	my ($opt , $val) = @_;

	if( ($opt eq "computer") && ($val) ){
		$computer = $val . "," . $computer;
	}
	else{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt",$CmdInfo{'cmdname'});
	}
}

sub handle_group_names
{
	my ($opt , $val) = @_;

	if( ($opt eq "group") && ($val) ){
		$groups = $groups . $val . ",";
	}
	else{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt",$CmdInfo{'cmdname'});
	}
}

sub handle_startup_directory
{
	my ($opt , $val) = @_;

	if($RepetitionCount{'handle_startup_directory'}) {
		#ThrowSyntaxErrorException("CQCLI_Repeat_Action","[$ActionDef and $opt ]", $CmdInfo{'cmdname'} );
	}else {
		$RepetitionCount{'handle_startup_directory'} = 1;
	}
	
	if( ($opt eq "startup") && ($val) ){
		$startupdir = $val;
	}
	else{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt",$CmdInfo{'cmdname'});
	}
}

sub handle_viewpath_name
{
	my ($opt , $val) = @_;

	if( ($opt eq "view") && ($val) ){
		$viewpathname=$val;
	}
	else{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt",$CmdInfo{'cmdname'});
	}
}

sub handle_log_name
{
	my ($opt , $val) = @_;

	if( ($opt eq "log") && ($val) ){
		$logname=$val;
	}
	else{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt",$CmdInfo{'cmdname'});
	}
}

sub parseForHelpUsage()
{
	my $ret = undef;

	my $helpRules = {'help|h|?'		=>	\&handle_help_usage	,
					 'man'			=>	\&handle_help_usage	};
	
	Getopt::Long::Configure('autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$helpRules} );
	Getopt::Long::Configure('default');

	return 1;
}

sub parseForInterActiveOption 
{
	my $ret = undef;
	my $askRules = {'ask!'	=>	\&handle_ask_mode	};
	Getopt::Long::Configure('posix_default','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$askRules} );
	Getopt::Long::Configure('default');

	return 1;
}

sub parseForLoginCredentials 
{
	my $ret = undef;
	my $CommonRules = {
		'database|dbs|d:s'					=>  \&handle_login_data			,
		'user|u:s'					=>	\&handle_login_data			,
		'schema|s|m|master:s'				=>	\&handle_login_data			,
		'password|p:s'						=>	\&handle_login_data			,
	};
	Getopt::Long::Configure( 'autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$CommonRules} );
	Getopt::Long::Configure('default');
	return 1;
}

sub parseForOtherOptions_tmexecute()
{
	my @tmpARGV = @ARGV;
	my $lmt = $#tmpARGV;
	my $msg = undef;

	for (my $cnt = 0; $cnt <= $lmt;$cnt++)  
	{	
		if($tmpARGV[$cnt] =~ /^-(id?$|i(d)?$)/i){
			$idSpecified = 1;
			$cnt = $cnt + 1;
			while(!($tmpARGV[$cnt] =~  /^(-)(.+)/)) {
				if(!($tmpARGV[$cnt])){
					goto DONE;
				}
				$ids = $ids . $tmpARGV[$cnt] . ",";
				$cnt = $cnt + 1;
			}
			--$cnt;
		}
		elsif($tmpARGV[$cnt] =~ /^-(computer?$|compute(r)?$|comput(er)?$|compu(ter)?$|comp(uter)?$|com(puter)?$|co(mputer)?$|c(omputer)?$)/i){
			$cnt = $cnt + 1;
			while(!($tmpARGV[$cnt] =~  /^(-)(.+)/)) {
				if(!($tmpARGV[$cnt])){
					goto DONE;
				}
				$computer = $computer . $tmpARGV[$cnt] . ",";
				$cnt = $cnt + 1;
			}
			--$cnt;
		}
		elsif($tmpARGV[$cnt] =~ /^-(group?$|g(roup)?$|gr(oup)?$|gro(up)?$|grou(p)?$)/i){
			$cnt = $cnt + 1;
			while(!($tmpARGV[$cnt] =~  /^(-)(.+)/)) {
				if(!($tmpARGV[$cnt])){
					goto DONE;
				}
				$groups = $groups . $tmpARGV[$cnt] . ",";
				$cnt = $cnt + 1;
			}
			--$cnt;
		}
		elsif($tmpARGV[$cnt] =~ /^-(commit?$|comm(it)?$|commi(t)?$)/i){
			handle_commit_option();
		}
		else{
			ThrowSyntaxErrorException('CQCLI_Invalid_Option',$tmpARGV[$cnt]);
		}
      }
	DONE:return 1;
}

sub parseForOtherOptions_tmcommit()
{
	my @tmpARGV = @ARGV;
	my $lmt = $#tmpARGV;
	my $msg = undef;

	for (my $cnt = 0; $cnt <= $lmt;$cnt++)  
	{	
		if($tmpARGV[$cnt] =~ /^-(id?$|i(d)?$)/i){
			$cnt = $cnt + 1;
			$idSpecified = 1;
			while(!($tmpARGV[$cnt] =~  /^(-)(.+)/)) {
				if(!($tmpARGV[$cnt])){
					goto DONE;
				}
				$ids = $ids . $tmpARGV[$cnt] . ",";
				$cnt = $cnt + 1;
			}
		}
		else {
			ThrowSyntaxErrorException('CQCLI_Invalid_Option',$tmpARGV[$cnt]);
		}
      }
	DONE:return 1;
}

sub parseForCoreOptions_tmexecute()
{
	my $ret = undef;
	my $CoreOptRules = {
		'iteration|i=s'   =>		\&handle_iteration_name, 
		'bname:s' 		=> 		\&handle_build_name,
		'brecord:s'   	=>		\&handle_build_id, 
		'view:s' 		=>   	\&handle_viewpath_name,
		'startup:s'     =>      \&handle_startup_directory
		};
	Getopt::Long::Configure( 'autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$CoreOptRules} );
	Getopt::Long::Configure('default');

	return 1;
}

sub parseForCoreOptions_tmcommit()
{
	my $ret = undef;
	my $CoreOptRules = {
		'iteration|i=s' =>		\&handle_iteration_name, 
		'bname:s' 		=> 		\&handle_build_name,
		'brecord:s'   	=>		\&handle_build_id, 
		'log:s' 		=>   	\&handle_log_name ,
		'startup:s'     =>      \&handle_startup_directory
		};
	Getopt::Long::Configure( 'autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$CoreOptRules} );
	Getopt::Long::Configure('default');

	return 1;
}

sub ValidateSessionInfo()
{
	
	if($isLoginOpt){
		ValidateLoginCredentials();
		return;
	}elsif($CliSession){
		return;
	}else{
		ValidateLoginCredentials();
		return;
	}

}
sub ValidateLoginCredentials()
{
	if(!($CmdLogin{'user'})){
		$CmdLogin{'user'} = GetUserName();
	}
	if(!($CmdLogin{'password'}) && $BLANK_MARKER_SET == 0){
		$CmdLogin{'password'} = GetPassword();
	}
	if(!($CmdLogin{'database'})){
		$CmdLogin{'database'} = GetDBName();
	}
	if(!($CmdLogin{'schema'})){
		$CmdLogin{'schema'} = GetSchemaName();
	}
	$CliSession=CreateSession(%CmdLogin);
	# is not a command scope session
	SetSessionScope(0);
}


sub createString
{
	my @valArr = @_;
	my $val = "";
	foreach my $keyval (@valArr) {
		$val = $val . $keyval;
		$val = $val . ",";
	}
	
	if($val eq ""){
		return undef;
	}

	return $val;
}

sub Parse_tmexecute()
{
	try {
		parseForHelpUsage();
		parseForInterActiveOption();
		parseForLoginCredentials();
		parseForCoreOptions_tmexecute();
		parseForOtherOptions_tmexecute();


		if(!($iteration)){
			ThrowSyntaxErrorException("CQCLI_Missing_Value","iteration",$CmdInfo{'cmdname'});
		}
		elsif($idSpecified == 0) {
			ThrowSyntaxErrorException("CQCLI_Missing_Value","ids",$CmdInfo{'cmdname'});
		}

		$JavaReqObj->put("iteration",$iteration);
		
		if($ids){
			$ids = substr($ids,0,-1);
			$JavaReqObj->put("id",$ids);
		}
		
		if($computer ne ""){
			$computer = substr($computer,0,-1);
			$JavaReqObj->put("computer",createString($computer));
		}
		else {
			$JavaReqObj->put("computer",undef);
		}
		
		if($groups ne "") {
			$groups = substr($groups,0,-1);
			$JavaReqObj->put("group",createString($groups));
		}
		else {
			$JavaReqObj->put("group",undef);
		}

		$JavaReqObj->put("commit",$commit);
		$JavaReqObj->put("view",$viewpathname);
		$JavaReqObj->put("bname",$buildname);
		$JavaReqObj->put("brecord",$buildid);
		$JavaReqObj->put("startup",$startupdir);
	}
	catch cqcli::Exception with {
		my $ex = shift;		
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_ErrJava_ParserUnknown");
		}
		else{
			ThrowParseException('CQCLI_Err_ParserUnknown',$@);
		}
		
    }
	finally {
		Getopt::Long::Configure('default');
	};

	return 1;
}

sub Parse_tmcommit()
{
	try {
		parseForHelpUsage();
		parseForInterActiveOption();
		parseForLoginCredentials();
		parseForCoreOptions_tmcommit();
		parseForOtherOptions_tmcommit();


		if(!($iteration)){
			ThrowSyntaxErrorException("CQCLI_Missing_Value","iteration",$CmdInfo{'cmdname'});
		}
		elsif($idSpecified == 0) {
			ThrowSyntaxErrorException("CQCLI_Missing_Value","ids",$CmdInfo{'cmdname'});
		}


		$JavaReqObj->put("iteration",$iteration);
		
		if($ids){
			$ids = substr($ids,0,-1);
			$JavaReqObj->put("id",$ids);
		}

		$JavaReqObj->put("log",$logname);
		$JavaReqObj->put("bname",$buildname);
		$JavaReqObj->put("brecord",$buildid);
		$JavaReqObj->put("startup",$startupdir);
	}
	catch cqcli::Exception with {
		my $ex = shift;		
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_ErrJava_ParserUnknown");
		}
		else{
			ThrowParseException('CQCLI_Err_ParserUnknown',$@);
		}
		
    }
	finally {
		Getopt::Long::Configure('default');
	};

	return 1;
}

sub Validate()
{
	try {
		ValidateSessionInfo();
	}
	catch cqcli::Exception with {
		my $ex = shift;		
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_ErrJava_ValidationUnknown");
		}
		else{
			ThrowValidationException('CQCLI_Err_ValidationUnknown',$@);
		}		
    }
	finally {	};

	return 1;
}

sub tmexecute
{
	my @tmpCmd = @_;
	my $req = undef;
	try {

		InitParser_tmexecute(@tmpCmd);
		Parse_tmexecute();
		Validate();
		$req =  GetJavaPayload("tmexecute",$JavaReqObj);
		SetRequestObject($req);
		return 1;
	}finally {
		CleanUpParser_tmexecute;
	};
}

sub tmcommit
{
	my @tmpCmd = @_;
	my $req = undef;

	try {

		InitParser_tmcommit(@tmpCmd);
		Parse_tmcommit();
		Validate();
		$req =  GetJavaPayload("tmcommit",$JavaReqObj);

		SetRequestObject($req);
		return 1;
	}finally {
		CleanUpParser_tmcommit();
	};
}



1;